@@ -1,3 +1,4 @@
+/* $Id: ClpMain.cpp 1489 2010-01-05 09:33:05Z forrest $ */
 // Copyright (C) 2002, International Business Machines
 // Corporation and others.  All Rights Reserved.
    
@@ -19,8 +20,7 @@ int boundary_sort3=10000;
 #include "CoinHelperFunctions.hpp"
 #include "CoinSort.hpp"
 // History since 1.0 at end
-#define CLPVERSION "1.06.00"
-
+#include "ClpConfig.h"
 #include "CoinMpsIO.hpp"
 #include "CoinFileIO.hpp"
 
@@ -52,6 +52,9 @@ int boundary_sort3=10000;
 #ifdef TAUCS_BARRIER
 #define FOREIGN_BARRIER
 #endif
+#ifdef MUMPS_BARRIER
+#define FOREIGN_BARRIER
+#endif
 
 static double totalTime=0.0;
 static bool maskMatches(const int * starts, char ** masks,
@@ -63,7 +66,7 @@ extern "C" {
 #if defined(_MSC_VER)
    __cdecl
 #endif // _MSC_VER
-   signal_handler(int whichSignal)
+   signal_handler(int /*whichSignal*/)
    {
       if (currentModel!=NULL) 
 	 currentModel->setMaximumIterations(0); // stop at next iterations
@@ -78,12 +81,13 @@ extern "C" {
 #endif
 
 int mainTest (int argc, const char *argv[],int algorithm,
-	      ClpSimplex empty, bool doPresolve,int switchOff,bool doVector);
+	      ClpSimplex empty, ClpSolve solveOptions,int switchOff,bool doVector);
 static void statistics(ClpSimplex * originalModel, ClpSimplex * model);
 static void generateCode(const char * fileName,int type);
 // Returns next valid field
 int CbcOrClpRead_mode=1;
 FILE * CbcOrClpReadCommand=stdin;
+extern int CbcOrClpEnvironmentIndex;
 
 int
 #if defined(_MSC_VER)
@@ -186,7 +190,8 @@ main (int argc, const char *argv[])
     // Hidden stuff for barrier
     int choleskyType = 0;
     int gamma=0;
-    int scaleBarrier=0;
+    parameters[whichParam(BARRIERSCALE,numberParameters,parameters)].setCurrentOption(2);
+    int scaleBarrier=2;
     int doKKT=0;
     int crossover=2; // do crossover unless quadratic
     
@@ -276,8 +281,13 @@ main (int argc, const char *argv[])
 	  std::cout<<"abcd without value (where expected) gives current value"<<std::endl;
 	  std::cout<<"abcd value sets value"<<std::endl;
 	  std::cout<<"Commands are:"<<std::endl;
-	  int maxAcross=5;
+	  int maxAcross=10;
 	  bool evenHidden=false;
+	  int printLevel = 
+	    parameters[whichParam(ALLCOMMANDS,
+				   numberParameters,parameters)].currentOptionAsInteger();
+	  int convertP[]={2,1,0};
+	  printLevel=convertP[printLevel];
 	  if ((verbose&8)!=0) {
 	    // even hidden
 	    evenHidden = true;
@@ -294,6 +304,7 @@ main (int argc, const char *argv[])
 	  int iType;
 	  for (iType=0;iType<4;iType++) {
 	    int across=0;
+	    int lengthLine=0;
             if ((verbose%4)!=0)
               std::cout<<std::endl;
 	    std::cout<<types[iType]<<std::endl;
@@ -301,16 +312,23 @@ main (int argc, const char *argv[])
               std::cout<<std::endl;
 	    for ( iParam=0; iParam<numberParameters; iParam++ ) {
 	      int type = parameters[iParam].type();
-	      if ((parameters[iParam].displayThis()||evenHidden)&&
+	      //printf("%d type %d limits %d %d display %d\n",iParam,
+	      //   type,limits[iType],limits[iType+1],parameters[iParam].displayThis());
+	      if ((parameters[iParam].displayThis()>=printLevel||evenHidden)&&
 		  type>=limits[iType]
 		  &&type<limits[iType+1]) {
 		if (!across) {
-                  if ((verbose&2)==0) 
-                    std::cout<<"  ";
-                  else
+                  if ((verbose&2)!=0) 
                     std::cout<<"Command ";
                 }
-                std::cout<<parameters[iParam].matchName()<<"  ";
+		int length = parameters[iParam].lengthMatchName()+1;
+		if (lengthLine+length>80) {
+		  std::cout<<std::endl;
+		  across=0;
+		  lengthLine=0;
+		}
+                std::cout<<" "<<parameters[iParam].matchName();
+		lengthLine += length ;
 		across++;
 		if (across==maxAcross) {
 		  across=0;
@@ -515,6 +533,9 @@ main (int argc, const char *argv[])
 	    case PFI:
 	      models[iModel].factorization()->setForrestTomlin(action==0);
 	      break;
+	    case FACTORIZATION:
+	      models[iModel].factorization()->forceOtherFactorization(action);
+	      break;
 	    case CRASH:
 	      doCrash=action;
 	      break;
@@ -540,7 +561,8 @@ main (int argc, const char *argv[])
 	      crossover=action;
 	      break;
 	    default:
-	      abort();
+	      //abort();
+	      break;
 	    }
 	  }
 	} else {
@@ -597,7 +619,7 @@ main (int argc, const char *argv[])
 		  }
 		}
 		if (tryIt) {
-		  model2 = ((ClpSimplexOther *) model2)->dualOfModel(fractionRow,fractionColumn);
+		  model2 = static_cast<ClpSimplexOther *> (model2)->dualOfModel(fractionRow,fractionColumn);
 		  if (model2) {
 		    printf("Dual of model has %d rows and %d columns\n",
 			   model2->numberRows(),model2->numberColumns());
@@ -706,8 +728,11 @@ main (int argc, const char *argv[])
 		  solveOptions.setSpecialOption(1,11); // switch off values
 	      } else if (method==ClpSolve::useBarrier||method==ClpSolve::useBarrierNoCross) {
 		int barrierOptions = choleskyType;
-		if (scaleBarrier)
-		  barrierOptions |= 8;
+		if (scaleBarrier) {
+		  if ((scaleBarrier&1)!=0) 
+		    barrierOptions |= 8;
+		  barrierOptions |= 2048*(scaleBarrier>>1);
+		}
 		if (doKKT)
 		  barrierOptions |= 16;
 		if (gamma)
@@ -733,9 +758,10 @@ main (int argc, const char *argv[])
               }
 #ifdef CLP_MULTIPLE_FACTORIZATIONS   
 	      int denseCode = parameters[whichParam(DENSE,numberParameters,parameters)].intValue();
-	      if (denseCode>=model2->numberRows()) {
-		model2->factorization()->goDense();
-	      }
+	      model2->factorization()->setGoDenseThreshold(denseCode);
+	      int smallCode = parameters[whichParam(SMALLFACT,numberParameters,parameters)].intValue();
+	      model2->factorization()->setGoSmallThreshold(smallCode);
+	      model2->factorization()->goDenseOrSmall(model2->numberRows());
 #endif
               try {
                 status=model2->initialSolve(solveOptions);
@@ -745,15 +771,14 @@ main (int argc, const char *argv[])
                 status=-1;
               }
               if (dualize) {
-                int returnCode=((ClpSimplexOther *) models+iModel)->restoreFromDual(model2);
+                int returnCode=static_cast<ClpSimplexOther *> (models+iModel)->restoreFromDual(model2);
 		if (model2->status()==3)
 		  returnCode=0;
                 delete model2;
 		if (returnCode&&dualize!=2) {
-		  CoinSighandler_t saveSignal=SIG_DFL;
 		  currentModel = models+iModel;
 		  // register signal handler
-		  saveSignal = signal(SIGINT,signal_handler);
+		  signal(SIGINT,signal_handler);
 		  models[iModel].primal(1);
 		  currentModel=NULL;
 		}
@@ -957,7 +982,7 @@ main (int argc, const char *argv[])
 		  // Go to canned file if just input file
 		  if (CbcOrClpRead_mode==2&&argc==2) {
 		    // only if ends .mps
-		    char * find = (char *)strstr(fileName.c_str(),".mps");
+		    char * find = const_cast<char *>(strstr(fileName.c_str(),".mps"));
 		    if (find&&find[4]=='\0') {
 		      find[1]='p'; find[2]='a';find[3]='r';
 		      FILE *fp=fopen(fileName.c_str(),"r");
@@ -1039,7 +1064,7 @@ main (int argc, const char *argv[])
 		bool deleteModel2=false;
 		ClpSimplex * model2 = models+iModel;
 		if (dualize&&dualize<3) {
-		  model2 = ((ClpSimplexOther *) model2)->dualOfModel();
+		  model2 = static_cast<ClpSimplexOther *> (model2)->dualOfModel();
 		  printf("Dual of model has %d rows and %d columns\n",
 			 model2->numberRows(),model2->numberColumns());
 		  model2->setOptimizationDirection(1.0);
@@ -1529,8 +1554,35 @@ main (int argc, const char *argv[])
 	      }
               int specialOptions = models[iModel].specialOptions();
               models[iModel].setSpecialOptions(0);
+	      ClpSolve solveOptions;
+	      ClpSolve::PresolveType presolveType;
+	      if (preSolve)
+		presolveType=ClpSolve::presolveOn;
+	      else
+		presolveType=ClpSolve::presolveOff;
+	      solveOptions.setPresolveType(presolveType,5);
+	      if (doSprint>=0||doIdiot>=0) {
+		if (doSprint>0) {
+		  // sprint overrides idiot
+		  solveOptions.setSpecialOption(1,3,doSprint); // sprint
+		} else if (doIdiot>0) {
+		  solveOptions.setSpecialOption(1,2,doIdiot); // idiot
+		} else {
+		  if (doIdiot==0) {
+		    if (doSprint==0)
+		      solveOptions.setSpecialOption(1,4); // all slack
+		    else
+		      solveOptions.setSpecialOption(1,9); // all slack or sprint
+		  } else {
+		    if (doSprint==0)
+		      solveOptions.setSpecialOption(1,8); // all slack or idiot
+		    else
+		      solveOptions.setSpecialOption(1,7); // initiative
+		  }
+		}
+	      }
 	      mainTest(nFields,fields,algorithm,models[iModel],
-		       (preSolve!=0),specialOptions,doVector!=0);
+		       solveOptions,specialOptions,doVector!=0);
 	    }
 	    break;
 	  case UNITTEST:
@@ -1547,7 +1599,15 @@ main (int argc, const char *argv[])
               int algorithm=-1;
               if (models[iModel].numberRows())
                 algorithm=7;
-	      mainTest(nFields,fields,algorithm,models[iModel],(preSolve!=0),specialOptions,doVector!=0);
+	      ClpSolve solveOptions;
+	      ClpSolve::PresolveType presolveType;
+	      if (preSolve)
+		presolveType=ClpSolve::presolveOn;
+	      else
+		presolveType=ClpSolve::presolveOff;
+	      solveOptions.setPresolveType(presolveType,5);
+	      mainTest(nFields,fields,algorithm,models[iModel],
+		       solveOptions,specialOptions,doVector!=0);
 	    }
 	    break;
 	  case FAKEBOUND:
@@ -1644,7 +1704,18 @@ clp watson.mps -\nscaling off\nprimalsimplex"
 		fp=stderr;
 		fprintf(fp,"\n");
 	      } else {
-		if (field[0]=='/'||field[0]=='\\') {
+                bool absolutePath;
+                if (dirsep=='/') {
+                  // non Windows (or cygwin)
+                  absolutePath=(field[0]=='/');
+                } else {
+                  //Windows (non cycgwin)
+                  absolutePath=(field[0]=='\\');
+                  // but allow for :
+                  if (strchr(field.c_str(),':'))
+                    absolutePath=true;
+                }
+		if (absolutePath) {
 		  fileName = field;
 		} else if (field[0]=='~') {
 		  char * environVar = getenv("HOME");
@@ -1686,6 +1757,7 @@ clp watson.mps -\nscaling off\nprimalsimplex"
 		int iRow;
 		int numberRows=models[iModel].numberRows();
 		int lengthName = models[iModel].lengthNames(); // 0 if no names
+		int lengthPrint = CoinMax(lengthName,8);
 		// in general I don't want to pass around massive
 		// amounts of data but seems simpler here
 		std::vector<std::string> rowNames =
@@ -1699,8 +1771,6 @@ clp watson.mps -\nscaling off\nprimalsimplex"
 		double * rowLower = models[iModel].rowLower();
 		double * rowUpper = models[iModel].rowUpper();
 		double primalTolerance = models[iModel].primalTolerance();
-		char format[6];
-		sprintf(format,"%%-%ds",CoinMax(lengthName,8));
                 bool doMask = (printMask!=""&&lengthName);
 		int * maskStarts=NULL;
 		int maxMasks=0;
@@ -1817,9 +1887,16 @@ clp watson.mps -\nscaling off\nprimalsimplex"
                       type=0;
                     if (type) {
                       fprintf(fp,"%7d ",iRow);
-                      if (lengthName)
-                        fprintf(fp,format,rowNames[iRow].c_str());
-                      fprintf(fp,"%15.8g        %15.8g\n",primalRowSolution[iRow],
+                      if (lengthName) {
+			const char * name = rowNames[iRow].c_str();
+			int n=strlen(name);
+			int i;
+			for (i=0;i<n;i++)
+			  fprintf(fp,"%c",name[i]);
+			for (;i<lengthPrint;i++)
+			  fprintf(fp," ");
+		      }
+                      fprintf(fp," %15.8g        %15.8g\n",primalRowSolution[iRow],
                               dualRowSolution[iRow]);
                     }
                   }
@@ -1848,9 +1925,16 @@ clp watson.mps -\nscaling off\nprimalsimplex"
                     type =0;
 		  if (type) {
 		    fprintf(fp,"%7d ",iColumn);
-		    if (lengthName)
-		      fprintf(fp,format,columnNames[iColumn].c_str());
-		    fprintf(fp,"%15.8g        %15.8g\n",
+		    if (lengthName) {
+		      const char * name = columnNames[iColumn].c_str();
+		      int n=strlen(name);
+		      int i;
+		      for (i=0;i<n;i++)
+			fprintf(fp,"%c",name[i]);
+		      for (;i<lengthPrint;i++)
+			fprintf(fp," ");
+		    }
+		    fprintf(fp," %15.8g        %15.8g\n",
 			    primalColumnSolution[iColumn],
 			    dualColumnSolution[iColumn]);
 		  }
@@ -1905,6 +1989,9 @@ clp watson.mps -\nscaling off\nprimalsimplex"
 	      
 	    }
 	    break;
+	  case ENVIRONMENT:
+	    CbcOrClpEnvironmentIndex=0;
+	    break;
 	  default:
 	    abort();
 	  }
@@ -1961,7 +2048,7 @@ static void breakdown(const char * name, int numberLook, const double * region)
     1.0,
     1.0e1,1.0e2,1.0e3,1.0e4,1.0e5,1.0e8,1.0e11,1.0e15,
     COIN_DBL_MAX};
-  int nRanges = (int) (sizeof(range)/sizeof(double));
+  int nRanges = static_cast<int> (sizeof(range)/sizeof(double));
   int * number = new int[nRanges];
   memset(number,0,nRanges*sizeof(int));
   int * numberExact = new int[nRanges];
@@ -2181,7 +2268,7 @@ static void statistics(ClpSimplex * originalModel, ClpSimplex * model)
   }
   printf("Column breakdown:\n");
   int k;
-  for (k=0;k<(int) (sizeof(cType)/sizeof(int));k++) {
+  for (k=0;k<static_cast<int> (sizeof(cType)/sizeof(int));k++) {
     printf("%d of type %s ",cType[k],cName[k].c_str());
     if (((k+1)%3)==0)
       printf("\n");
@@ -2189,7 +2276,7 @@ static void statistics(ClpSimplex * originalModel, ClpSimplex * model)
   if ((k%3)!=0)
     printf("\n");
   printf("Row breakdown:\n");
-  for (k=0;k<(int) (sizeof(rType)/sizeof(int));k++) {
+  for (k=0;k<static_cast<int> (sizeof(rType)/sizeof(int));k++) {
     printf("%d of type %s ",rType[k],rName[k].c_str());
     if (((k+1)%3)==0)
       printf("\n");
@@ -2601,7 +2688,7 @@ static void statistics(ClpSimplex * originalModel, ClpSimplex * model)
 	if (numberPossible>1&&!numberIntegers) {
 	  //printf("possible %d - but no integers\n",numberPossible);
 	}
-	if (numberPossible>1&&(numberIntegers||true)) {
+	if (numberPossible>1&&(numberIntegers||false)) {
 	  // 
 	  printf("possible %d - %d integers\n",numberPossible,numberIntegers);
 	  int lastLook=iLook;
@@ -2995,4 +3082,5 @@ static void generateCode(const char * fileName,int type)
   1.03.03 June 13 2006.  For clean up after dual perturbation
   1.04.01 June 26 2007.  Lots of changes but I got lazy
   1.05.00 June 27 2007.  This is trunk so when gets to stable will be 1.5
+  1.11.00 November 5 2009 (Guy Fawkes) - OSL factorization and better ordering
  */
